<?
require_once(dirname(__FILE__).'/../../blibFrame.lib.php');
require_once(dirname(__FILE__).'/../blibIO/blibFile.class.php');

//file: blibXMLParser.class.php
/*************************************************************\
*	XmlParser.
*	XML input parsing class
*	Base class for any operations relying on XML, as input.
\*************************************************************/
	
		
//parse type (EXPAT events handlers||xml_parse_into_struct)
define('XMLPARSER_PARSE_BY_STRUCT', 1);
define('XMLPARSER_PARSE_BY_EVENT' , 2);
		
class blibXMLParser extends blibClass{
  var $hFile;                   //pointer to blibFile object
	var $a_parsers = array();	//array of XML parser objects
	var $o_parser;						//XML current parser object
	var $s_data = '';					//XML input string for parsing.
														//This property provides ability to
														//parse raw strings as well as file
														//contents.
	var $a_values = array();	//data array produced by 
														//xml_parse_into_struct()
	var $a_indexes = array();	//locations/pointers array produced 
														//by xml_parse_into_struct()
	var $i_parse_type = XMLPARSER_PARSE_BY_STRUCT; //Parse type
														//take look at class constants.
	var $a_parser_options = array(//XML parser options
											'XML_OPTION_SKIP_WHITE'			=>	1,
											'XML_OPTION_CASE_FOLDING'		=>	0,
											'XML_OPTION_TARGET_ENCODING'=>	'ISO-8859-1'
											);
  //constructor
  function init() {
  	$this->parser_create();
  }	
	
	function parse($s_file = '', $m_parser = 0, $o_object = NULL) {
		$m_return = true;
		
    //read input file if required
  	$this->hFile = new blibFile();
    if($data = $this->hFile->contentGet($s_file,true)) {
      $this->s_data = $data;	
    }
    
    
		//avoid parsing empty strings		
		if ($this->s_data == '') return $m_return;

		//define parser object
		if(!is_resource($m_parser)) {
			$this->o_parser = (array_key_exists($m_parser,$this->a_parsers))?
													$this->a_parsers[$m_parser]:$this->a_parsers[0];
		} else {
			$this->o_parser = $m_parser;
		}

		//for use of parser inside of class
		//this declaration must be presented in the parse function,
		//if defined in any other function , parser<->object bound
		//goes out of scope as soon as function's call exits.
		if(is_object($o_object) && !is_null($o_object)) {
			xml_set_object($this->o_parser, &$o_object);
		}	else {
			xml_set_object($this->o_parser, &$this);	
		}
		
		switch($this->i_parse_type) {	
			//use xml_parse_into_struct function
			case XMLPARSER_PARSE_BY_STRUCT:
				xml_parse_into_struct($this->o_parser, $this->s_data, 
								&$this->a_values, &$this->a_indexes) or $m_return = $this->print_error();
				$m_return = ($m_return == 1)?
					array($this->a_values,$this->a_indexes):$m_return;
				break;
			//use J.Clark's EXPAT compatible event handlers
			case XMLPARSER_PARSE_BY_EVENT:
				$this->set_handlers($this->o_parser);
				xml_parse($this->o_parser,$this->s_data) or $m_return = print_error();
				break;
		}
		return $m_return;
	}
	
	function parser_create($b_force_new = 0) {
		$m_return = true;
		//if doesn't exist create new parser handler
		if (!isset($this->o_parser)||$b_force_new) {
			$this->o_parser = xml_parser_create();
			$this->a_parsers[] = $this->o_parser;
		} else {
			return $this->o_parser; 
		}		

		//un/strip any unneccessary white sapces
		xml_parser_set_option($this->o_parser,XML_OPTION_SKIP_WHITE,
													$this->a_parser_options['XML_OPTION_SKIP_WHITE']) 
														or $m_return  = $this->print_error();
		
		//upercasing is XML specifications violation (XML is case sensitive)
		//so case_folding is turned off (by this class)
		xml_parser_set_option($this->o_parser,XML_OPTION_CASE_FOLDING,
													$this->a_parser_options['XML_OPTION_CASE_FOLDING'])
														or $m_return  = $this->print_error();

		//seting chars encoding
		xml_parser_set_option($this->o_parser,XML_OPTION_TARGET_ENCODING,
													$this->a_parser_options['XML_OPTION_TARGET_ENCODING'])
														or $m_return  = $this->print_error();

		
		return ($m_return)?$this->o_parser:$m_return;
	}
	
	//events handlers
	function event_start($parser, $name, $attr) {
		//empty
	}

	function event_close($parser, $name) {
		//empty
	}

	function event_cdata($parser, $data) {
		//empty
	}

	function event_pi($parser, $type, $data) {
		//empty
	}

	function event_default($parser, $data) {
		//empty
	}

	function set_handlers($o_parser) {
		$b_return = true;
		$this->assign_handlers($o_parser,'el','event_start','event_close') 
																		or $b_return = $this->print_error();
		$this->assign_handlers($o_parser,'cdata','event_cdata')
																		or $b_return = $this->print_error();
		$this->assign_handlers($o_parser,'cdata','event_cdata')
																		or $b_return = $this->print_error();
		$this->assign_handlers($o_parser,'pi','event_pi')
																		or $b_return = $this->print_error();
		$this->assign_handlers($o_parser,'default','event_default')
																		or $b_return = $this->print_error();
		return $b_return;
	}

	function assign_handlers($o_parser, $s_type, $s_handler = '', $s_handler1 = '') {
		switch(strtolower($s_type) ) {
			case 'element_handler':
				xml_set_element_handler($o_parser,$s_handler,$s_handler1);
			case 'el':
				xml_set_element_handler($o_parser,$s_handler,$s_handler1);
				break;
			case 'character_data_handler':
				xml_set_character_data_handler($o_parser,$s_handler);
			case 'cdata':
				xml_set_character_data_handler($o_parser,$s_handler);
				break;
			case 'processing_instruction_handler':
				xml_set_processing_instruction_handler($o_parser, $s_handler);
			case 'pi':
				xml_set_processing_instruction_handler($o_parser, $s_handler);
				break;
			case 'default_handler':
				xml_set_default_handler($o_parser, $s_handler);
			case 'default':
				xml_set_default_handler($o_parser, $s_handler);
				break;
			default:
					return false;
				break;
		}
		return true;
	}

	//destructor
	function destroy() {
		//free XML parser
		foreach($this->a_parsers as $val) {
			xml_parser_free($val);
		}
		//unset all class properties
		foreach(get_object_vars($this) as $key=>$val) {
			unset($key);
		}
	} 
	
	/***********************************************************\
	*	Other Methods
	*	Auxiliary functions.
	\***********************************************************/
	
	//displays errors in understandable fashion
	function print_error() {
    $this->errorThrow(get_class($this),(sprintf("XML Error: %s at line %d",
									xml_error_string(xml_get_error_code($this->o_parser)),
									xml_get_current_line_number($this->o_parser)
									)),__FILE__,__LINE__);
		return false;
	} 	

}

//file:blibXMLConfParser.class.php
class blibXMLConfParser extends blibXMLParser{
  var $confData;
  //constructor
  function init() {
  	parent::init();
  }	

  function parseConfig($inFile) {
    $this->hFile = new blibFile($inFile);
    preg_match("/<GIRA_XML>.*?<\/GIRA_XML>/si"
                ,$this->hFile->contentGet()
                ,$conf_data);
    $this->s_data = $conf_data[0];
    $this->parse();

    $this->confData = array(); //array of all XML-vocabulary sections
    foreach ($this->a_values as $v ) {
      if ($v['level'] >=2) {
        if ( $v['tag']=='SECT'&&$v['type']=='open' ) {
          $sec_name = $v['attributes']['name'];
          $this->confData[$sec_name] = array();
        }elseif ( $v['tag']=='TEXT'&&$v['type']=='open' ) {
          $str_name = $v['attributes']['name'];
          $this->confData[$sec_name][$str_name] = '';
        }elseif ( $v['tag']=='VAR'&&$v['type']=='complete' ) {
          $this->confData[$sec_name][$str_name] = (isset($v['value'])?$v['value']:'');
        }
      }
    }
		$this->confData['raw_paths'] = $this->confData['paths'];
    foreach ($this->confData['paths'] as $k=>$v) {
      $this->confData['paths'][$k] = "http".(isset($HTTP_SERVER_VARS["HTTPS"])&&$HTTP_SERVER_VARS["HTTPS"]=="on"?"s":"")."://".$_SERVER['HTTP_HOST'].$v;
    }
    return $this->confData;
  }

  function getConfig() {
  	return $this->confData;
  }
}

//file: blibXMLSectionParser.class.php
class blibXMLSectionParser extends blibXMLParser{
  var $confData;
  //constructor
  function init() {
  	parent::init();
  }	

  function parseSections($inFile) {
    $this->hFile = new blibFile($inFile);
    preg_match("/<XML>.*?<\/XML>/si"
                ,$this->hFile->contentGet()
                ,$conf_data);
		$this->s_data = $conf_data[0];
    $this->parse();

    $this->confData = array(); //array of all XML-vocabulary sections
    foreach ($this->a_values as $v ) {
      if ($v['level'] >=2) {
        if ( $v['tag']=='SECT'&&$v['type']=='open' ) {
          $sec_name = $v['attributes']['name'];
          $this->confData[$sec_name] = array();
        }elseif ( $v['tag']=='TEXT'&&$v['type']=='open' ) {
          $str_name = $v['attributes']['name'];
          $this->confData[$sec_name][$str_name] = '';
        }elseif ( $v['tag']=='VAR'&&$v['type']=='complete' ) {
          $this->confData[$sec_name][$str_name] = $v['value'];
        }
      }
    }
    return $this->confData;
  }

  function getConfig() {
  	return $this->confData;
  }
}

?>